class Interval:
    MIN1 = 60
    MIN3 = 3 * MIN1
    MIN5 = 5 * MIN1
    MIN15 = 15 * MIN1
    MIN30 = 30 * MIN1
    HOUR1 = 60 * MIN1
    HOUR2 = 2 * HOUR1
    HOUR4 = 4 * HOUR1
    HOUR6 = 6 * HOUR1
    HOUR8 = 8 * HOUR1
    HOUR12 = 12 * HOUR1
    DAY1 = 24 * HOUR1
    DAY3 = 3 * DAY1
    WEEK1 = 7 * DAY1
    MON1 = 30 * DAY1
    YEAR1 = 12 * MON1



class LenghtNotMatch(Exception):
    def __init__(self, message="Length of Two Series Do not Match"):            
        # Call the base class constructor with the parameters it needs
        super().__init__(message)


import os
import math

import pandas as pd
import matplotlib.pyplot as plt

from .constants.interval import Interval
from .constants.definitions import * 


def resampling_based_on_time_frame(source_df: pd.DataFrame, source_interval: Interval, destination_interval: Interval) -> pd.DataFrame:
    """_summary_

    Args:
        source_df (pd.DataFrame): _description_
        source_interval (Interval): _description_
        destination_interval (Interval): _description_

    Returns:
        _type_: _description_
    """
    
    # TODO: throw error if the dataframe is smaller than the step
    
    step = int(destination_interval / source_interval)
    print(step)
    df = source_df.copy()
    df['open'] = df.open.rolling(step).agg(lambda w: w.iloc[0]).shift(-step + 1)
    df['high'] = df.high.rolling(step).max().shift(-step + 1)
    df['low'] = df.low.rolling(step).min().shift(-step + 1)
    df['close'] = df.close.rolling(step).agg(lambda w: w.iloc[-1]).shift(-step + 1)
    df['volume'] = df.volume.rolling(step).sum().shift(-step + 1)

    df = df.dropna() # drop last nan rows
    df = df[(0 == df.timestamp % destination_interval)] # keep destination interval rows and drop extra rows 
    df = df.reset_index(drop=True) # reset index to zero and drop previous indices
    
    return df


from finbright_utils.fin_statistics import correlation, resample
from finbright_utils.constants.interval import Interval
import pandas as pd
import yfinance as yf
import datetime 
import numpy as np

min1_data_aapl = yf.download("AAPL", start="2023-01-31", end="2023-02-01", interval = "1m")["Adj Close"]
min1_data_msf = yf.download("MSFT", start="2023-01-31", end="2023-02-01", interval = "1m")["Adj Close"]
corr = correlation(min1_data_aapl, min1_data_msf)
print(corr)
# min1_data.index = pd.to_datetime(min1_data.index)
# min1_data.drop(["Adj Close"], axis=1, inplace=True)
# min1_data.rename(columns={'Open': 'open',
#                 'High': 'high',
#                 'Low': 'low',
#                 'Close': 'close',
#                 'Volume': 'volume',
#                     },
#         inplace=True, errors='raise')
# min1_data["timestamp"] = min1_data.index
# min1_data.reset_index(drop=True, inplace=True)
# # min1_data.index = pd.to_datetime(min1_data.index)
# # print(min1_data["Datetime"])
# min1_data['timestamp'] = min1_data["timestamp"].values.astype(np.int64) // 10 ** 9

# day1_data = resample.resampling_based_on_time_frame(min1_data,  Interval.MIN1, Interval.MIN5)

# print(day1_data)


import pandas as pd
from statsmodels.tsa.stattools import coint
from finbright_utils import resample
from .constants.errors import LenghtNotMatch


def correlation(X1: pd.Series, X2: pd.Series) -> int:
    """_summary_

    Args:
        data (pd.DataFrame): A close price dataframe of two coins

    Returns:
        int: correlation coeffiecient between close prices of two coins
    """
    if len(X1) == len(X2):

        corr_coefficient = X1.corr(X2)

        return round(corr_coefficient, 2)

    else:
        raise (LenghtNotMatch)


def cointegration(X1: pd.Series, X2: pd.Series) -> int:
    """_summary_

    Args:
        data (pd.DataFrame): A close price dataframe of two coins

    Returns:
        int: cointegation coeffiecient between close prices of two coins
    """
    if len(X1) == len(X2):

        score, pvalue, _ = coint(X1, X2)

        return pvalue

    else:
        raise (LenghtNotMatch)
